@@ -7,7 +7,7 @@ from contextlib import contextmanager
 from datetime import datetime
 from typing import ContextManager, Iterator, Sequence, Union
 
-from coba.pipes import Pipe, Filter, Sink, NullIO, ConsoleIO
+from coba.pipes import Pipes, Filter, Sink, NullSink, ConsoleSink, Identity
 from coba.exceptions import CobaException
 
 class Logger(ABC):
@@ -27,7 +27,7 @@ class Logger(ABC):
     @abstractmethod
     def log(self, message: Union[str,Exception]) -> 'ContextManager[Logger]':
         """Log a message or exception to the sink.
-        
+
         Args:
             message: The message or exception that should be logged.
 
@@ -52,7 +52,7 @@ class NullLogger(Logger):
     """A logger which writes nothing."""
 
     def __init__(self) -> None:
-        self._sink = NullIO()
+        self._sink = NullSink()
 
     @contextmanager
     def _context(self) -> 'Iterator[Logger]':
@@ -75,9 +75,9 @@ class NullLogger(Logger):
 class BasicLogger(Logger):
     """A Logger with flat hierarchy and separate begin/end messages."""
 
-    def __init__(self, sink: Sink[str] = ConsoleIO()):
+    def __init__(self, sink: Sink[str] = ConsoleSink()):
         """Instantiate a BasicLogger.
-        
+
         Args:
             sink: The sink to write to (by default console).
         """
@@ -98,7 +98,7 @@ class BasicLogger(Logger):
             raise
         finally:
             self.log(message + f" {outcome}")
-    
+
     @contextmanager
     def _time_context(self, message:str) -> 'Iterator[Logger]':
         self.log(message)
@@ -113,7 +113,7 @@ class BasicLogger(Logger):
             raise
         else:
             self.log(message + f" ({round(time.time()-self._starts.pop(),2)} seconds) (completed)")
-        
+
     @property
     def sink(self) -> Sink[str]:
         return self._sink
@@ -132,9 +132,9 @@ class BasicLogger(Logger):
 class IndentLogger(Logger):
     """A Logger with indentation hierarchy and a single timed log with total runtime."""
 
-    def __init__(self, sink: Sink[str] = ConsoleIO()):
+    def __init__(self, sink: Sink[str] = ConsoleSink()):
         """Instantiate an IndentLogger.
-        
+
         Args:
             sink: The sink to write the logs to (by default console).
         """
@@ -155,7 +155,7 @@ class IndentLogger(Logger):
     @contextmanager
     def _time_context(self, message:str) -> 'Iterator[Logger]':
 
-        # we don't have all the information we need to write 
+        # we don't have all the information we need to write
         # our message but we want to save our place in line
         # we also level our message before entering context
         place_in_line = len(self._messages)
@@ -212,18 +212,18 @@ class DecoratedLogger(Logger):
 
     def __init__(self, pre_decorators: Sequence[Filter], logger: Logger, post_decorators: Sequence[Filter]):
         """Instantiate DecoratedLogger.
-        
+
         Args:
             pre_decorators: A sequence of decorators to be applied before the base logger.
             logger: The base logger we are decorating.
             post_decorators: A sequence of decorators to be applied after the base logger.
         """
 
-        self._pre_decorator   = Pipe.join(pre_decorators)
+        self._pre_decorator   = Pipes.join(*pre_decorators) if pre_decorators else Identity()
         self._post_decorators = post_decorators
         self._logger          = logger
         self._original_sink   = self._logger.sink
-        self._logger.sink     = Pipe.join(post_decorators, self._original_sink)
+        self._logger.sink     = Pipes.join(*post_decorators, self._original_sink)
 
     @property
     def sink(self) -> Sink[str]:
@@ -232,7 +232,7 @@ class DecoratedLogger(Logger):
     @sink.setter
     def sink(self, sink: Sink[str]):
         self._original_sink = sink
-        self._logger.sink   = Pipe.join(self._post_decorators, sink)
+        self._logger.sink   = Pipes.join(*self._post_decorators, sink)
 
     def log(self, message: Union[str,Exception]) -> 'ContextManager[Logger]':
         return self._logger.log(self._pre_decorator.filter(message))
@@ -267,7 +267,7 @@ class ExceptLog(Filter[Union[str,Exception],str]):
             return log
         elif isinstance(log, CobaException):
             return str(log)
-        else: 
+        else:
             tb  = ''.join(traceback.format_tb(log.__traceback__))
             msg = ''.join(traceback.TracebackException.from_exception(log).format_exception_only())
             return f"Unexpected exception:\n\n{tb}\n  {msg}"
\ No newline at end of file
