@@ -5,6 +5,9 @@ from coba.environments import OpenmlSimulation, SerializedSimulation, Supervised
 from coba.environments import Sort, Scale, Cycle, Shuffle, Take, Identity, Where
 from coba.contexts     import DiskCacher, NullCacher, IndentLogger, NullLogger, BasicLogger
 
+CobaRegistry.register("range" , range)
+CobaRegistry.register("zip"   , zip  )
+
 CobaRegistry.register("Null"   , NullSink   )
 CobaRegistry.register("Disk"   , DiskSink   )
 CobaRegistry.register("Http"   , HttpSource )
@@ -14,8 +17,8 @@ CobaRegistry.register("BasicLogger" , BasicLogger )
 CobaRegistry.register("IndentLogger", IndentLogger)
 CobaRegistry.register("NullLogger"  , NullLogger  )
 
-CobaRegistry.register("DiskCacher", DiskCacher  )
-CobaRegistry.register("NullCacher", NullCacher  )
+CobaRegistry.register("DiskCacher", DiskCacher)
+CobaRegistry.register("NullCacher", NullCacher)
 
 CobaRegistry.register("OpenmlSimulation"    , OpenmlSimulation    )
 CobaRegistry.register("SerializedSimulation", SerializedSimulation)
